function PHI = GreedyLinear(A,Ahat,xbar,beta,lambda,T)
% This function computes adversary's Greedy policy when the environment evolves by linear dynamic.
% This is the results in Theorem 3. 
% It returns a function handle of policy function phi. 
% In this code, we shift vector's time index by 1. For example, 
% xbar(t+1,t'+1) corresponds to xbar_{t'|t} in paper. However, matrix 
% dimension remain the same for all of the matrices in Theorem 1. 
%
%
% Input:
% A: (p+1)-by-(p+1) matrix in the environment dynamics:
% x_{t+1} = F(x_t, u_t, w_t) := A x_t + B (u_t + w_t)
%
% Ahat: (p+1)-by-(p+1) forecast matrix, the forecaster's AR(p) model
%
% xbar: (T+1)-by-(T+1) matrix, scalar version of adversarial reference target.
% xbar(t+1,t'+1) corresponds to xbar_{t'|t} in paper. For xbar(t+1,t'+1) 
% that is: t < 0 or t>=t' or t=T, xbar(t+1,t'+1) should be set to 0.
%
% beta: (T+1)-by-(T+1) matrix, nonzero entry of Q matrix, beta(t+1,t'+1)
% corresponds to the nonzero entry in the quadratic cost matrix Q_{t'|t} in 
% paper. For beta(t+1,t'+1) that is: t<0 or t>=t' or t=T, beta(t+1,t'+1)
% should be set to 0.
%
% lambda>0: adversarial effort parameter
%
% T: prediction time horizon
%
%
% Output:
% phi: function handle for ploicy function. One can call it as:
%
% ut = phi(t,xt)
%
% xt is the (p+1)-by-1 state vector, t is time index

% retrieve p
p = length(A(:,1)) - 1;

% define B
B = zeros(p+1,1);
B(2) = 1;

% initialize phi, start at 0
% phi{t+1} corresponds to phi_t
phi = cell(T,1);
% compute phi_t
for t = 0 : T-2
	sum1 = zeros(1,p+1);
	sum2 = 0;
	% set Q_{t+1:T | t}
    % Q(:,:,t'+1) corresponds to Q_{t'|t}
	Q = zeros(p+1,p+1,T+1);
	Q(2,2,t+2+1:T+1) = beta(t+1+1, t+2+1:T+1);

	% set bfxbar, only the second entry will be used
    % bfbar(:,t'+1) corresponds to barbfx_{t'|t}
	bfxbar = zeros(p+1,T+1);
	bfxbar(2,t+2+1:T+1) = xbar(t+1+1, t+2+1:T+1);

	% compute the sums in Thm 3
	for tp = t+2 : T
		sum1 = sum1 + (Ahat^((tp + 1) - (t + 1 + 1)) * B)' * Q(:,:,tp+1) * Ahat^((tp + 1) - (t + 1 + 1));
		sum2 = sum2 + (Ahat^((tp + 1) - (t + 1 + 1)) * B)' * Q(:,:,tp+1) * bfxbar(:,tp+1);
	end
	phi{t+1} = @(z) -(sum1 * A * z - sum2) / (lambda + sum1 * B);
end
phi{T} = @(z) 0;
% restructure phi
PHI = @(t,xt) phi{t+1}(xt);
end