function ustar = ILQR(F, u, xtau, tau, L, T, Ahat, xbar, beta, lambda, maxiter, tol)
% This function is to implement ILQR in each MPC step.
% In this code, we shift vector's time index by 1. For example, 
% xbar(t+1,t'+1) corresponds to xbar_{t'|t} in paper.
%
% Inputs:
% F(x,u,w,mode) is a function handle with three modes for environment dynamic \bfx_{t+1} = F(\bfx_t, u_t, w_t, 1)
% when mode = 1, F(x,u,w,1) returns F(x,u,w)
% when mode = 2, F(x,u,w,2) returns DxF(x,u,w), DxF_{ij} corresponds to dF_i/dxj
% when mode = 3, F(x,u,w,3) returns DuF(x,u,w), DuF_{i1} corresponds to dF_i/du 
% 
% u: 1-by-(L+1 - tau) array for initial attack action sequence
%
% xtau: current state value
%
% tau: current time index
%
% L: time horizon minus 1 for the ILQR, should be set to L(tau)
%
% T: prediction time horizon
%
% Ahat: (p+1)-by-(p+1) matrix, the forecaster's AR(p) model
%
% xbar: (T+1)-by-(T+1) matrix, scalar version of adversarial target.
% xbar(t+1,t'+1) corresponds to xbar_{t'|t} in paper. For xbar(t+1,t'+1) 
% that is: t < 0 or t>=t' or t=T, xbar(t+1,t'+1) should be set to 0.
%
% beta: (T+1)-by-(T+1) matrix, nonzero entry of Q matrix, beta(t+1,t'+1)
% corresponds to the nonzero entry in the quadratic cost matrix Q_{t'|t} in 
% paper. For beta(t+1,t'+1) that is: t<0 or t>=t' or t=T, beta(t+1,t'+1)
% should be set to 0.
%
% lambda>0: adversarial effort parameter
%
% maxiter: maximum iteration for ILQR
%
%
% Output:
% ustar: 1-by-(L+1 - tau) array for attack action sequence obtained by ILQR: u_t, t = tau, tau+1, ..., L+1-1

% retrieve p
p = length(Ahat(1,:))-1;

% A 1-by-(L+1 - tau) array for ustar_t, t = tau, tau+1, ..., L	
utilde = zeros(1, L+1 - tau);

% A 1-by-(L+1 - tau) array for \delta u_t, t = tau, tau+1, ..., L		
udel = zeros(1, L+1 - tau);

% A (p+1)-by-(L+1 - tau + 1) array for \bfx_t, t = tau, tau+1, ..., L+1
xtilde = zeros(p+1, L+1 - tau + 1);
xtilde(:,1) = xtau;

% A (p+1)-by-(L+1 - tau + 1) array for \delta \bfx_t, t = tau, tau+1, ..., L+1
xdel = zeros(p+1, L+1 - tau + 1);

for ite = 1 : maxiter

	% simulate the trajectory
	for t = 1 : L+1 - tau
		xtilde(:,t+1) = F(xtilde(:,t), utilde(t), 0,1);
	end

	% a (p+1)-by-(p+1)-by-(L(tau) - tau) array for: P_{\tau}, ..., P_{L(\tau)+1}
	P = zeros(p+1,p+1,L-tau+2);	

	% a (p+1)-by-(L(tau) - tau) array for: \bfq_{\tau}, ..., \bfq_{L(\tau)+1}
	q = zeros(p+1,L-tau+2);	

	% a (p+1)-by-(p+1) array to transfer beta to Q matrix
	Q = zeros(p+1,p+1);	

	% a (p+1)-by-1 array to transfer scalar xbar to vector xbar
	bfxbar = zeros(p+1,1);
	bfxbar(1) = 1; 

	% compute P_{L(tau)+1}, q_{L(tau)+1}
	for tp = L+2 : T
		Q(2,2) = beta(L+2,tp+1); 
		bfxbar(2) = xbar(L+2,tp+1);
		P(:,:,L-tau+2) = P(:,:,L-tau+2) + (Ahat^(tp - (L+1)))'*Q*(Ahat^(tp - (L+1)));
		q(:,L-tau+2) = q(:,L-tau+2) + 2 * (Ahat^(tp - (L+1)))'*Q*(Ahat^(tp - (L+1))*xtilde(:,L-tau+2) - bfxbar);
	end

	% compute P_s, q_s for s = L,...,tau 
	for s = L-tau+1 : -1: 1 % the s here is shifted by tau - 1 from the original
		% compute Jocabian matrices:
		Dx = F(xtilde(:,s), utilde(s), 0, 2);
		Du = F(xtilde(:,s), utilde(s), 0, 3);

		P(:,:,s) = Dx'*(eye(p+1) + lambda^(-1)*P(:,:,s+1)*Du*Du')^(-1)*P(:,:,s+1)*Dx';
		q(:,s) = Dx'*q(:,s+1)-(Du'*P(:,:,s+1)*Dx)'*(Du'*q(:,s+1)+2*lambda*utilde(s))/(lambda+Du'*P(:,:,s+1)*Du);

		% the original s
		sr = s + tau - 1;

		for tp = sr+1 : T
			Q(2,2) = beta(sr+1,tp+1); 
			bfxbar(2) = xbar(sr+1,tp+1);

			P(:,:,s) = P(:,:,s) + (Ahat^(tp - sr))'*Q*(Ahat^(tp - sr));
			q(:,s) = q(:,s) + 2 * (Ahat^(tp - sr))'*Q*(Ahat^(tp - sr)*xtilde(:,s) - bfxbar);
		end

	end

	% compute \delta u
	for s = 1 : L+1-tau
		% compute Jocabian matrices:
		Dx = F(xtilde(:,s), utilde(s), 0, 2);
		Du = F(xtilde(:,s), utilde(s), 0, 3);

		udel(s) = -(2 * Du' * P(:,:,s+1) * Dx * xdel(:,s) + Du' * q(:,s+1) + 2 * lambda * utilde(s))/(2*(lambda + Du' * P(:,:,s+1) * Du));
		xdel(:,s+1) = Dx * xdel(:,s) + Du * udel(s);
	end

	utilde = utilde + udel;
	if udel'*udel/(L+1 - tau) < tol
		break
	end
end

ustar = utilde;

end
