function [xhat, x, u0] = Pred(F, Ahat, x0, T, u, w)
% this is a function to generate forecaster's prediction and simulation evolution of system with attack sequence
% 
% inputs: 
% F(x,u,w,mode) is a function handle with three modes for dynamic \bfx_{t+1} = F(\bfx_t, u_t, w_t, 1)
% when mode = 1, F(x,u,w,1) returns F(x,u,w)
% when mode = 2, F(x,u,w,2) returns DxF(x,u,w), DxF_{ij} corresponds to dF_i/dxj
% when mode = 3, F(x,u,w,3) returns DuF(x,u,w), DuF_{i1} corresponds to dF_i/du 
%
% Ahat: (p+1)-by-(p+1) matrix, the forecaster's AR(p) model
%
% x0: the scalar initial value
%
% T: prediction time horizon
%
% u: 1-by-T array for action sequence u_t, t = 0, 1, ... , T-1
%
% w: 1-by-T the noise sequence w_t, t = 0,1,...,T-1
%
% outputs:
% xhat: (T+1)-by-(T+1) matrix for prediction sequences, xhat(t+1,t'+1) corresponds to xhat_{t'|t}
% x: (p+1)-by-(T+1) matrix for state value: \bfx_t, t = 0, 1, ..., T 

% retrieve p
p = length(Ahat(:,1)) - 1;

xhat = zeros(T+1, T+1);
x = zeros(p+1, T+1);
x(1,:) = ones(1, T+1);
x(2,1) = x0;

% C is a column vector to pick out x_t from \bfx_t
C = zeros(p+1,1);
C(2) = 1;

for t = 0 : T-1
	% prediction
	for tp = t+1 : T
		xhat(t+1, tp+1) = C' * Ahat^(tp - t)*x(:, t+1);
	end
	% evolve system
	x(:, t+2) = F(x(:, t+1), u(t+1), w(t+1), 1);
end
u0 = u;
end
