% this code is used to generate plot 'figexper1_Qs.eps' and Table 1 for the toy example in section 4.1.
%
% A, B is the environment dynamic;
% Ahat is forecast matrix
% x0 is the scalar initial state
% T is the time horizon
% bar is the attack reference target (value)
% lambdatilde is \tilde\lambda in paper

clear;

% set up the parameters
A = [1, 0; 1, 0.5];
Ahat = [1, 0; 0.9, 0.6];
B = [0; 1];
x0 = 0;
T = 10;
lambdatilde = 10^(-1);
bar = 1; 

% set lambda to balance the different sum number
lambda = @(beta) lambdatilde*sum(sum(beta))/T;

% set a function handle for the environment dynamic
F = @(x, u, w, mode) A * x + B * u + B * w;

% set the beta matrix beta and attack target xbar when attack target is to change tomorrow's prediction
beta_tom = diag(ones(1,T),1);
xbar_tom = bar * beta_tom;

% set the beta matrix beta and attack target xbar when attack target is to change final day's prediction
beta_final = zeros(T+1, T+1);
beta_final(1:T, T+1) = ones(T, 1);
xbar_final = bar * beta_final;

% set the beta matrix beta and attack target xbar when attack target is to change all prediction
beta_all = triu(ones(T+1,T+1),1);
xbar_all = bar * ones(T+1,1) * ones(1,T+1) .* beta_all;

% used fixed random seed to generate noise sequence
rng(1);

% set noise sequence 
w = normrnd(0, 0.1, [1, T]);	

% when there is no attack, compute predictions, state values, and a zero control sequence u_non
[xhat_non, x_non, u_non] = Pred(F, Ahat, x0, T, zeros(1, T), w);	


% when the attack target is to change tomorrow's prediction	
% compute the policy functions of LQR and Greedy
PHI_lqr_tom = LQRStateAttack(A,Ahat,xbar_tom,beta_tom,lambda(beta_tom),T);
PHI_G_tom = GreedyLinear(A,Ahat,xbar_tom,beta_tom,lambda(beta_tom),T);

% compute state, attack sequence and prediction sequence under LQR and Greedy policy
[xhat_lqr_tom, x_lqr_tom, u_lqr_tom] = PredAttPlo(F, Ahat, x0, T, @(t,x) PHI_lqr_tom(t,x), w);
[xhat_G_tom, x_G_tom, u_G_tom] = PredAttPlo(F, Ahat, x0, T, @(t,x) PHI_G_tom(t,x), w);	

% compute the realized cost when use LQR, Greedy and no attack
cost_lqr_tom = Cost(Ahat, xbar_tom, beta_tom, x_lqr_tom, u_lqr_tom, lambda(beta_tom), T);
cost_G_tom = Cost(Ahat, xbar_tom, beta_tom, x_G_tom, u_G_tom, lambda(beta_tom), T);
cost_non_tom = Cost(Ahat, xbar_tom, beta_tom, x_non, u_non, lambda(beta_tom), T);	


% when attack target is to change final day's prediction	
% compute policy functions of LQR anf Greedy
PHI_lqr_final = LQRStateAttack(A,Ahat,xbar_final,beta_final,lambda(beta_final),T);
PHI_G_final = GreedyLinear(A,Ahat,xbar_final,beta_final,lambda(beta_final),T);	

% compute state, attack sequence and prediction sequence uder LQR and Greedy policy
[xhat_lqr_final, x_lqr_final, u_lqr_final] = PredAttPlo(F, Ahat, x0, T, @(t,x) PHI_lqr_final(t,x), w);
[xhat_G_final, x_G_final, u_G_final] = PredAttPlo(F, Ahat, x0, T, @(t,x) PHI_G_final(t,x), w);	

% compute the realized cost when use LQR, Greedy and no attack
cost_lqr_final = Cost(Ahat, xbar_final, beta_final, x_lqr_final, u_lqr_final, lambda(beta_final), T);
cost_G_final = Cost(Ahat, xbar_final, beta_final, x_G_final, u_G_final, lambda(beta_final), T);
cost_non_final = Cost(Ahat, xbar_final, beta_final, x_non, u_non, lambda(beta_final), T);	


% when attack target is to change all prediction	
% compute policy functions of LQR and Greedy
PHI_lqr_all = LQRStateAttack(A,Ahat,xbar_all,beta_all,lambda(beta_all),T);
PHI_G_all = GreedyLinear(A,Ahat,xbar_all,beta_all,lambda(beta_all),T);	

% compute state, attack sequence and prediction sequence under LQR and Greedy policy
[xhat_lqr_all, x_lqr_all, u_lqr_all] = PredAttPlo(F, Ahat, x0, T, @(t,x) PHI_lqr_all(t,x), w);
[xhat_G_all, x_G_all, u_G_all] = PredAttPlo(F, Ahat, x0, T, @(t,x) PHI_G_all(t,x), w);	

% compute the realized cost when use LQR, Greedy and no attack
cost_lqr_all = Cost(Ahat, xbar_all, beta_all, x_lqr_all, u_lqr_all, lambda(beta_all), T);
cost_G_all = Cost(Ahat, xbar_all, beta_all, x_G_all, u_G_all, lambda(beta_all), T);
cost_non_all = Cost(Ahat, xbar_all, beta_all, x_non, u_non, lambda(beta_all), T);	
	


% generate the plot in figure 1
figtoy1 = figure

% Plot predictions, state values when there is no attack
ax1 = subplot(2,2,1);
% plot predictions
for t = 0 : T-1
	p1 = plot([t+1:T], xhat_non(t+1, t+2:T+1), '-b.'); hold on;
end
% plot x_t, t = 0, 1, ..., T
p2 = plot([0:T], x_non(2,:), '-m.');

xlabel('time');
title(['no attack']);
legend([p1, p2], {'forecast $\hat x_{t'' \mid t} $ ', 'env state $x_t$'}, 'Location', 'southeast','interpreter','latex');



% Plot when attack target is to change tomorrow's prediction
ax2 = subplot(2,2,2);
% plot the predictions sequence, environment state sequence, attack action sequence
[p1, p2, p3] = Predplot(xhat_lqr_tom, x_lqr_tom, u_lqr_tom, T);
% plot attack reference target
p4 = plot([1:T], diag(xbar_tom, 1), '-g.');
% highlight target prediction \hat x_{t+1 | t}
p5 = plot([1:T], diag(xhat_lqr_tom, 1), '*b');
lgd = legend([p3, p4], {' LQR $u_t$', 'target $\bar x_{t+1 \mid t}$'}, 'Location', 'southeast','interpreter','latex');
str = ['attack predictions for tomorrows: $\hat x_{t+1 \mid t}$'];
title(str,'interpreter','latex');


% when attack target is to change final day's prediction
ax3 = subplot(2,2,3);
% plot the predictions, environment state sequence, attack action sequence
[p1, p2, p3] = Predplot(xhat_lqr_final, x_lqr_final, u_lqr_final, T);
% plot attack reference target
p4 = plot([1:T], xbar_final(T, T+1)*ones(1,T), '-g.');
% highlight target prediction \hat x_{t' | t}
for t = 0 : T-1
	p5 = plot([T], xhat_lqr_final(t+1, T+1), 'b*'); hold on;
end
lgd = legend([p3, p4], {'LQR $u_t$', 'target $\bar x_{T \mid t}$'}, 'Location', 'northwest','interpreter','latex');
str = ['attack predictions for final day: $\hat x_{T \mid t}$'];
title(str,'interpreter','latex');


% when attack target is to change all prediction
ax4 = subplot(2,2,4);
% plot the predictions, environment state sequence, attack action sequence
[p1, p2, p3] = Predplot(xhat_lqr_all, x_lqr_all, u_lqr_all, T);
% plot attack referencetarget 
p4 = plot([1:T], diag(xbar_all, 1), '-g.');
% highlight target prediction \hat x_{T | t}
for t = 0 : T-1
	p5 = plot([t+1:T], xhat_lqr_all(t+1, t+2:T+1), 'b*'); hold on;
end
lgd = legend([p3, p4], {'LQR $u_t$', 'target $\bar x_{t'' \mid t}$'}, 'Location', 'best','interpreter','latex');
str = ['attack all predictions: $\hat x_{t'' \mid t}$'];
title(str,'interpreter','latex');

linkaxes([ax1,ax2,ax3,ax4],'xy');


% generate table 1
cost_tom_non = Cost(Ahat, xbar_tom, beta_tom, x_non, u_non, lambda(beta_tom), T);
cost_tom_utom = Cost(Ahat, xbar_tom, beta_tom, x_lqr_tom, u_lqr_tom, lambda(beta_tom), T);
cost_tom_ufinal = Cost(Ahat, xbar_tom, beta_tom, x_lqr_final, u_lqr_final, lambda(beta_tom), T);
cost_tom_uall = Cost(Ahat, xbar_tom, beta_tom, x_lqr_all, u_lqr_all, lambda(beta_tom), T);

cost_final_non = Cost(Ahat, xbar_final, beta_final, x_non, u_non, lambda(beta_final), T);
cost_final_utom = Cost(Ahat, xbar_final, beta_final, x_lqr_tom, u_lqr_tom, lambda(beta_final), T);
cost_final_ufinal = Cost(Ahat, xbar_final, beta_final, x_lqr_final, u_lqr_final, lambda(beta_final), T);
cost_final_uall = Cost(Ahat, xbar_final, beta_final, x_lqr_all, u_lqr_all, lambda(beta_final), T);

cost_all_non = Cost(Ahat, xbar_all, beta_all, x_non, u_non, lambda(beta_all), T);
cost_all_utom = Cost(Ahat, xbar_all, beta_all, x_lqr_tom, u_lqr_tom, lambda(beta_all), T);
cost_all_ufinal = Cost(Ahat, xbar_all, beta_all, x_lqr_final, u_lqr_final, lambda(beta_all), T);
cost_all_uall = Cost(Ahat, xbar_all, beta_all, x_lqr_all, u_lqr_all, lambda(beta_all), T);

table = [cost_tom_utom,cost_tom_ufinal,cost_tom_uall,cost_tom_non;
cost_final_utom,cost_final_ufinal,cost_final_uall,cost_final_non;
cost_all_utom,cost_all_ufinal,cost_all_uall,cost_all_non];

disp(round(table,2))


saveas(figtoy1,'figexper1_Qs','epsc');


% function used to generate plot for prediction \hat x_{t' | t} (t = 0, ..., T-1, t' = t+1, ... T), 
% state sequence x_t (t = 0, ..., T) , control sequence u_t (t = 0, ..., T-1)
function [p1, p2, p3] = Predplot(xhat, x, u, T)
for t = 0 : T-1
	p1 = plot([t+1:T], xhat(t+1, t+2:T+1), '-b.'); hold on;
end
p2 = plot([0:T], x(2,:), '-m.');
p3 = plot([0:T-1], u, '-r.');

xlabel('time');

end
